@@ -3,9 +3,10 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2009-2019, University of Amsterdam
+    Copyright (c)  2009-2023, University of Amsterdam
                               VU University Amsterdam
                               CWI, Amsterdam
+                              SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -41,6 +42,7 @@
             expand_goal/4,              % +Goal0, ?Pos0, -Goal, -Pos
             var_property/2,             % +Var, ?Property
 
+            '$including'/0,
             '$expand_closure'/3         % +GoalIn, +Extra, -GoalOut
           ]).
 
@@ -108,12 +110,17 @@ expand_term(Term, Pos0, [], Pos) :-
     !,
     atomic_pos(Pos0, Pos).
 expand_term(Term, Pos0, Expanded, Pos) :-
+    setup_call_cleanup(
+        '$push_input_context'(expand_term),
+        expand_term_keep_source_loc(Term, Pos0, Expanded, Pos),
+        '$pop_input_context').
+
+expand_term_keep_source_loc(Term, Pos0, Expanded, Pos) :-
     b_setval('$term', Term),
     prepare_directive(Term),
     '$def_modules'([term_expansion/4,term_expansion/2], MList),
     call_term_expansion(MList, Term, Pos0, Term1, Pos1),
-    expand_terms(expand_term_2, Term1, Pos1, Term2, Pos),
-    rename(Term2, Expanded),
+    expand_terms(expand_term_2, Term1, Pos1, Expanded, Pos),
     b_setval('$term', []).
 
 %!  prepare_directive(+Directive) is det.
@@ -434,14 +441,19 @@ diff3(>,  H1, T1, _H2, T2, Diff) :-
 %   branches claim the variable to  be   fresh,  we  can consider it
 %   fresh.
 
-merge_variable_info([]).
-merge_variable_info([Var=State|States]) :-
+merge_variable_info(State) :-
+    catch(merge_variable_info_(State),
+          error(uninstantiation_error(Term),_),
+          throw(error(goal_expansion_error(bound, Term), _))).
+
+merge_variable_info_([]).
+merge_variable_info_([Var=State|States]) :-
     (   get_attr(Var, '$var_info', CurrentState)
     ->  true
     ;   CurrentState = (-)
     ),
     merge_states(Var, State, CurrentState),
-    merge_variable_info(States).
+    merge_variable_info_(States).
 
 merge_states(_Var, State, State) :- !.
 merge_states(_Var, -, _) :- !.
@@ -473,13 +485,18 @@ save_variable_info([Var|Vars], [Var=State|States]):-
     ),
     save_variable_info(Vars, States).
 
-restore_variable_info([]).
-restore_variable_info([Var=State|States]) :-
+restore_variable_info(State) :-
+    catch(restore_variable_info_(State),
+          error(uninstantiation_error(Term),_),
+          throw(error(goal_expansion_error(bound, Term), _))).
+
+restore_variable_info_([]).
+restore_variable_info_([Var=State|States]) :-
     (   State == (-)
     ->  del_attr(Var, '$var_info')
     ;   put_attr(Var, '$var_info', State)
     ),
-    restore_variable_info(States).
+    restore_variable_info_(States).
 
 %!  var_property(+Var, ?Property)
 %
@@ -757,25 +774,26 @@ expand_meta(Spec, G0, P0, G, P, M, MList, Term, Done) :-
     functor(Spec, _, Arity),
     functor(G0, Name, Arity),
     functor(G1, Name, Arity),
-    f_pos(P0, ArgPos0, P, ArgPos),
+    f_pos(P0, ArgPos0, G1P, ArgPos),
     expand_meta(1, Arity, Spec,
-                G0, ArgPos0, Eval,
+                G0, ArgPos0, Eval, EvalPos,
                 G1,  ArgPos,
                 M, MList, Term, Done),
-    conj(Eval, G1, G).
+    conj(Eval, EvalPos, G1, G1P, G, P).
 
-expand_meta(I, Arity, Spec, G0, ArgPos0, Eval, G, [P|PT], M, MList, Term, Done) :-
+expand_meta(I, Arity, Spec, G0, ArgPos0, Eval, EvalPos, G, [P|PT],
+            M, MList, Term, Done) :-
     I =< Arity,
     !,
     arg_pos(ArgPos0, P0, PT0),
     arg(I, Spec, Meta),
     arg(I, G0, A0),
     arg(I, G, A),
-    expand_meta_arg(Meta, A0, P0, EvalA, A, P, M, MList, Term, Done),
+    expand_meta_arg(Meta, A0, P0, EvalA, EPA, A, P, M, MList, Term, Done),
     I2 is I + 1,
-    expand_meta(I2, Arity, Spec, G0, PT0, EvalB, G, PT, M, MList, Term, Done),
-    conj(EvalA, EvalB, Eval).
-expand_meta(_, _, _, _, _, true, _, [], _, _, _, _).
+    expand_meta(I2, Arity, Spec, G0, PT0, EvalB,EPB, G, PT, M, MList, Term, Done),
+    conj(EvalA, EPA, EvalB, EPB, Eval, EvalPos).
+expand_meta(_, _, _, _, _, true, _, _, [], _, _, _, _).
 
 arg_pos(List, _, _) :- var(List), !.    % no position info
 arg_pos([H|T], H, T) :- !.              % argument list
@@ -818,7 +836,7 @@ extended_pos(F-T,
 extended_pos(Pos, N, Pos) :-
     '$print_message'(warning, extended_pos(Pos, N)).
 
-%!  expand_meta_arg(+MetaSpec, +Arg0, +ArgPos0, -Eval,
+%!  expand_meta_arg(+MetaSpec, +Arg0, +ArgPos0, -Eval, -EvalPos,
 %!                  -Arg, -ArgPos, +ModuleList, +Term, +Done) is det.
 %
 %   Goal expansion for a meta-argument.
@@ -827,11 +845,11 @@ extended_pos(Pos, N, Pos) :-
 %           functions on such positions.  This requires proper
 %           position management for function expansion.
 
-expand_meta_arg(0, A0, PA0, true, A, PA, M, MList, Term, Done) :-
+expand_meta_arg(0, A0, PA0, true, _, A, PA, M, MList, Term, Done) :-
     !,
     expand_goal(A0, PA0, A1, PA, M, MList, Term, Done),
     compile_meta_call(A1, A, M, Term).
-expand_meta_arg(N, A0, P0, true, A, P, M, MList, Term, Done) :-
+expand_meta_arg(N, A0, P0, true, _, A, P, M, MList, Term, Done) :-
     integer(N), callable(A0),
     replace_functions(A0, true, _, M),
     !,
@@ -842,11 +860,11 @@ expand_meta_arg(N, A0, P0, true, A, P, M, MList, Term, Done) :-
     compile_meta_call(A2, A3, M, Term),
     term_variables(A0, VL),
     remove_arg_pos(A3, PA2, M, VL, Ex, A, P).
-expand_meta_arg(^, A0, PA0, true, A, PA, M, MList, Term, Done) :-
+expand_meta_arg(^, A0, PA0, true, _, A, PA, M, MList, Term, Done) :-
     !,
     expand_setof_goal(A0, PA0, A, PA, M, MList, Term, Done).
-expand_meta_arg(S, A0, _PA0, Eval, A, _PA, M, _MList, _Term, _Done) :-
-    replace_functions(A0, Eval, A, M), % TBD: pass positions
+expand_meta_arg(S, A0, PA0, Eval, EPA, A, PA, M, _MList, _Term, _Done) :-
+    replace_functions(A0, PA0, Eval, EPA, A, PA, M),
     (   Eval == true
     ->  true
     ;   same_functor(A0, A)
@@ -1164,9 +1182,7 @@ map_functions(I0, Arity, Term0, LPos0, Term, LPos, Eval, EP, Ctx) :-
     map_functions(I, Arity, Term0, APT0, Term, APT, Eval1, EP1, Ctx),
     conj(Eval0, EP0, Eval1, EP1, Eval, EP).
 
-conj(true, X, X) :- !.
-conj(X, true, X) :- !.
-conj(X, Y, (X,Y)).
+%!  conj(+G1, +P1, +G2, +P2, -G, -P)
 
 conj(true, _, X, P, X, P) :- !.
 conj(X, P, true, _, X, P) :- !.
@@ -1519,64 +1535,6 @@ member_eq(E, [H|T]) :-
     ;   member_eq(E, T)
     ).
 
-                 /*******************************
-                 *            RENAMING          *
-                 *******************************/
-
-:- multifile
-    prolog:rename_predicate/2.
-
-rename(Var, Var) :-
-    var(Var),
-    !.
-rename(end_of_file, end_of_file) :- !.
-rename(Terms0, Terms) :-
-    is_list(Terms0),
-    !,
-    '$current_source_module'(M),
-    rename_preds(Terms0, Terms, M).
-rename(Term0, Term) :-
-    '$current_source_module'(M),
-    rename(Term0, Term, M),
-    !.
-rename(Term, Term).
-
-rename_preds([], [], _).
-rename_preds([H0|T0], [H|T], M) :-
-    (   rename(H0, H, M)
-    ->  true
-    ;   H = H0
-    ),
-    rename_preds(T0, T, M).
-
-rename(Var, Var, _) :-
-    var(Var),
-    !.
-rename(M:Term0, M:Term, M0) :-
-    !,
-    (   M = '$source_location'(_File, _Line)
-    ->  rename(Term0, Term, M0)
-    ;   rename(Term0, Term, M)
-    ).
-rename((Head0 :- Body), (Head :- Body), M) :-
-    !,
-    rename_head(Head0, Head, M).
-rename((:-_), _, _) :-
-    !,
-    fail.
-rename(Head0, Head, M) :-
-    rename_head(Head0, Head, M).
-
-rename_head(Var, Var, _) :-
-    var(Var),
-    !.
-rename_head(M:Term0, M:Term, _) :-
-    !,
-    rename_head(Term0, Term, M).
-rename_head(Head0, Head, M) :-
-    prolog:rename_predicate(M:Head0, M:Head).
-
-
                  /*******************************
                  *      :- IF ... :- ENDIF      *
                  *******************************/
