@@ -566,7 +566,9 @@ initialise_error(E) :-
 
 initialise_prolog :-
     '$clean_history',
+    apply_defines,
     apple_setup_app,                            % MacOS cwd/locale setup for swipl-win
+    init_optimise,
     '$run_initialization',
     argv_prolog_files(Files, ScriptMode),
     '$load_system_init_file',                   % -F file
@@ -595,6 +597,60 @@ initialise_prolog :-
         )
     ).
 
+apply_defines :-
+    '$cmd_option_val'(defines, Defs),
+    apply_defines(Defs).
+
+apply_defines([]).
+apply_defines([H|T]) :-
+    apply_define(H),
+    apply_defines(T).
+
+apply_define(Def) :-
+    sub_atom(Def, B, _, A, '='),
+    !,
+    sub_atom(Def, 0, B, _, Flag),
+    sub_atom(Def, _, A, 0, Value0),
+    (   '$current_prolog_flag'(Flag, Value0, _Scope, Access, Type)
+    ->  (   Access \== write
+        ->  '$permission_error'(set, prolog_flag, Flag)
+        ;   text_flag_value(Type, Value0, Value)
+        ),
+	set_prolog_flag(Flag, Value)
+    ;   (   atom_number(Value0, Value)
+	->  true
+	;   Value = Value0
+	),
+	create_prolog_flag(Flag, Value, [warn_not_accessed])
+    ).
+apply_define(Def) :-
+    atom_concat('no-', Flag, Def),
+    !,
+    set_user_boolean_flag(Flag, false).
+apply_define(Def) :-
+    set_user_boolean_flag(Def, true).
+
+set_user_boolean_flag(Flag, Value) :-
+    current_prolog_flag(Flag, Old),
+    !,
+    (   Old == Value
+    ->  true
+    ;   set_prolog_flag(Flag, Value)
+    ).
+set_user_boolean_flag(Flag, Value) :-
+    create_prolog_flag(Flag, Value, [warn_not_accessed]).
+
+text_flag_value(integer, Text, Int) :-
+    atom_number(Text, Int),
+    !.
+text_flag_value(float, Text, Float) :-
+    atom_number(Text, Float),
+    !.
+text_flag_value(term, Text, Term) :-
+    term_string(Term, Text, []),
+    !.
+text_flag_value(_, Value, Value).
+
 :- if(current_prolog_flag(apple,true)).
 apple_set_working_directory :-
     (   expand_file_name('~', [Dir]),
@@ -621,6 +677,12 @@ apple_setup_app :-
 :- endif.
 apple_setup_app.
 
+init_optimise :-
+    current_prolog_flag(optimise, true),
+    !,
+    use_module(user:library(apply_macros)).
+init_optimise.
+
 opt_attach_packs :-
     current_prolog_flag(packs, true),
     !,
