@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2011-2022, VU University Amsterdam
+    Copyright (c)  2011-2023, VU University Amsterdam
                               CWI, Amsterdam
                               SWI-Prolog Solutions b.v.
     All rights reserved.
@@ -49,6 +49,7 @@
 		       eof_action(oneof([eof_code,error,reset])),
 		       buffer(oneof([full,line,false])),
 		       close_on_abort(boolean),
+		       close_on_exec(boolean),
 		       lock(oneof([none,read,shared,write,exclusive])),
                        reposition(boolean),
 		       wait(boolean),
@@ -154,7 +155,9 @@
 		     ]).
 :- predicate_options(system:create_prolog_flag/3, 3,
 		     [ access(oneof([read_write,read_only])),
-		       type(oneof([boolean,atom,integer,float,term])),
+		       type(( oneof([boolean,atom,integer,float,term])
+                            ; compound(oneof(list(atom)))
+                            )),
 		       keep(boolean)
 		     ]).
 :- predicate_options(system:qsave_program/2, 2,
