@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2012-2021, VU University Amsterdam
+    Copyright (c)  2012-2023, VU University Amsterdam
                               CWI, Amsterdam
                               SWI-Prolog Solutions b.v.
     All rights reserved.
@@ -79,7 +79,27 @@ library lets you inspect installed   packages,  install packages, remove
 packages, etc. It is complemented by   the  built-in attach_packs/0 that
 makes installed packages available as libraries.
 
+To make changes to a package:
+  * Clone the git repo,t go into the repo and run:
+    `?- pack_install(.).`
+    This builds the pack locally and creates a symlink to make it available.
+  * `?- pack_rebuild(package_name).`
+    This runs `make distclean` and `make` with the right environment. It will also
+    write a file `buildendv.sh` that you can source to get the environment
+    for running a normal `make` (this is done only if there is a `configure`
+    step; i.e., if there is a `configure.in` or `configure`.
+  * The build process also supports `cmake`.
+
+Once you have made the changes, you should edit the `pack.pl` file
+to change the `version` item. After updating the git repo, issue
+a `pack_install(package_name, [upgrade(true), test(true), rebuild(make)])`
+to cause the repository to refresh. You can simulate the full
+installation process by removing all the build files in the package
+(including any in submodules), running pack_install/1, and then
+running pack_install using a =|file://|= URL.
+
 @see    Installed packages can be inspected using =|?- doc_browser.|=
+@see    library(build/tools)
 @tbd    Version logic
 @tbd    Find and resolve conflicts
 @tbd    Upgrade git packages
@@ -460,14 +480,17 @@ search_info(download(_)).
 %
 %   Install a package.  Spec is one of
 %
+%     * A package name.  This queries the package repository
+%       at http://www.swi-prolog.org
 %     * Archive file name
 %     * HTTP URL of an archive file name.  This URL may contain a
 %       star (*) for the version.  In this case pack_install asks
 %       for the directory content and selects the latest version.
 %     * GIT URL (not well supported yet)
-%     * A local directory name given as =|file://|= URL or `'.'`
-%     * A package name.  This queries the package repository
-%       at http://www.swi-prolog.org
+%     * A local directory name given as =|file://|= URL
+%     * `'.'`, in which case a relative symlink is created to the
+%       current directory (all other options for Spec make a copy
+%       of the files).
 %
 %   After resolving the type of package,   pack_install/2 is used to
 %   do the actual installation.
@@ -563,12 +586,14 @@ version_options(_, _, []).
 %
 %   Select from available packages.
 
-pack_select_candidate(Pack, [Version-_|_], Options,
+pack_select_candidate(Pack, [AtomVersion-_|_], Options,
                       [already_installed(pack(Pack, Installed))|Options]) :-
     current_pack(Pack),
     pack_info(Pack, _, version(InstalledAtom)),
     atom_version(InstalledAtom, Installed),
+    atom_version(AtomVersion, Version),
     Installed @>= Version,
+    in_explicit_pack_dir(Pack, Options),
     !.
 pack_select_candidate(Pack, Available, Options, OptsOut) :-
     option(url(URL), Options),
@@ -586,13 +611,14 @@ pack_select_candidate(Pack, [Version-[URL]|_], Options,
     !,
     confirm(install_from(Pack, Version, git(URL)), yes, Options).
 pack_select_candidate(Pack, [Version-[URL]|More], Options,
-                      [url(URL), inquiry(true)]) :-
+                      [url(URL), inquiry(true) | Upgrade]) :-
     (   More == []
     ->  !
     ;   true
     ),
     confirm(install_from(Pack, Version, URL), yes, Options),
-    !.
+    !,
+    add_upgrade(Pack, Upgrade).
 pack_select_candidate(Pack, [Version-URLs|_], Options,
                       [url(URL), inquiry(true)|Rest]) :-
     maplist(url_menu_item, URLs, Tagged),
@@ -603,16 +629,32 @@ pack_select_candidate(Pack, [Version-URLs|_], Options,
     (   Choice == cancel
     ->  fail
     ;   Choice = git(URL)
-    ->  Rest = [git(true)]
+    ->  Rest = [git(true)|Upgrade]
     ;   Choice = URL,
-        Rest = []
-    ).
+        Rest = Upgrade
+    ),
+    add_upgrade(Pack, Upgrade).
+
+add_upgrade(Pack, Options) :-
+    current_pack(Pack),
+    !,
+    Options = [upgrade(true)].
+add_upgrade(_, []).
 
 url_menu_item(URL, git(URL)=install_from(git(URL))) :-
     git_url(URL, _),
     !.
 url_menu_item(URL, URL=install_from(URL)).
 
+%!  in_explicit_pack_dir(+Pack, +Options) is semidet.
+%
+%   True when Pack is installed in the explicit target directory.
+
+in_explicit_pack_dir(Pack, Options) :-
+    option(package_directory(Root), Options),
+    current_pack(Pack, PackDir),
+    file_directory_name(PackDir, Parent),
+    same_file(Parent, Root).
 
 %!  pack_install(+Name, +Options) is det.
 %
@@ -632,6 +674,9 @@ url_menu_item(URL, URL=install_from(URL)).
 %     existing and writable directory.  If that doesn't exist find
 %     locations where it can be created and prompt the user to do
 %     so.
+%     * insecure(+Boolean)
+%     When `true` (default `false`), do not perform any checks on SSL
+%     certificates when downloading using `https`.
 %     * interactive(+Boolean)
 %     Use default answer without asking the user if there
 %     is a default action.
@@ -746,6 +791,10 @@ pack_install(Name, _, Options) :-
     current_pack(Name, Dir),
     option(upgrade(false), Options, false),
     \+ pack_is_in_local_dir(Name, Dir, Options),
+    (   option(package_directory(_), Options)
+    ->  in_explicit_pack_dir(Name, Options)
+    ;   true
+    ),
     print_message(error, pack(already_installed(Name))),
     pack_info(Name),
     print_message(information, pack(remove_with(Name))),
@@ -1003,10 +1052,12 @@ pack_install_from_url(Scheme, URL, PackTopDir, Pack, Options) :-
     pack_download_dir(PackTopDir, DownLoadDir),
     download_file(URL, Pack, DownloadBase, Options),
     directory_file_path(DownLoadDir, DownloadBase, DownloadFile),
+    (   option(insecure(true), Options, false)
+    ->  TLSOptions = [cert_verify_hook(ssl_verify)]
+    ;   TLSOptions = []
+    ),
     setup_call_cleanup(
-        http_open(URL, In,
-                  [ cert_verify_hook(ssl_verify)
-                  ]),
+        http_open(URL, In, TLSOptions),
         setup_call_cleanup(
             open(DownloadFile, write, Out, [type(binary)]),
             copy_stream_data(In, Out),
@@ -1054,7 +1105,7 @@ pack_url_file(URL, FileID) :-
 
 :- public ssl_verify/5.
 
-%!  ssl_verify(+SSL, +ProblemCert, +AllCerts, +FirstCert, +Error)
+%   ssl_verify(+SSL, +ProblemCert, +AllCerts, +FirstCert, +Error)
 %
 %   Currently we accept  all  certificates.   We  organise  our  own
 %   security using SHA1 signatures, so  we   do  not  care about the
@@ -1101,11 +1152,11 @@ download_scheme(https) :-
 pack_post_install(Pack, PackDir, Options) :-
     post_install_foreign(Pack, PackDir, Options),
     post_install_autoload(PackDir, Options),
-    '$pack_attach'(PackDir).
+    attach_packs(PackDir, [duplicate(warning)]).
 
 %!  pack_rebuild(+Pack) is det.
 %
-%   Rebuilt possible foreign components of Pack.
+%   Rebuild possible foreign components of Pack.
 
 pack_rebuild(Pack) :-
     current_pack(Pack, PackDir),
@@ -1369,11 +1420,15 @@ info_file(todo(_),   'todo').
 git_url(URL, Pack) :-
     uri_components(URL, Components),
     uri_data(scheme, Components, Scheme),
+    nonvar(Scheme),                         % must be full URL
     uri_data(path, Components, Path),
     (   Scheme == git
     ->  true
     ;   git_download_scheme(Scheme),
         file_name_extension(_, git, Path)
+    ;   git_download_scheme(Scheme),
+        catch(git_ls_remote(URL, _, [refs(['HEAD']), error(_)]), _, fail)
+    ->  true
     ),
     file_base_name(Path, PackExt),
     (   file_name_extension(Pack, git, PackExt)
@@ -1456,12 +1511,16 @@ github_release_url(URL, Pack, Version) :-
     uri_data(scheme, Components, Scheme),
     download_scheme(Scheme),
     uri_data(path, Components, Path),
-    atomic_list_concat(['',_Project,Pack,archive,File], /, Path),
+    github_archive_path(Archive,Pack,File),
+    atomic_list_concat(Archive, /, Path),
     file_name_extension(Tag, Ext, File),
     github_archive_extension(Ext),
     tag_version(Tag, Version),
     !.
 
+github_archive_path(['',_User,Pack,archive,File],Pack,File).
+github_archive_path(['',_User,Pack,archive,refs,tags,File],Pack,File).
+
 github_archive_extension(tgz).
 github_archive_extension(zip).
 
@@ -1478,7 +1537,7 @@ version_tag_prefix('').
 :- public
     atom_version/2.
 
-%!  atom_version(?Atom, ?Version)
+%   atom_version(?Atom, ?Version)
 %
 %   Translate   between   atomic   version   representation   and   term
 %   representation.  The  term  representation  is  a  list  of  version
