@@ -731,6 +731,7 @@ safe_primitive(system:upcase_atom(_,_)).
 safe_primitive(system:is_list(_)).
 safe_primitive(system:memberchk(_,_)).
 safe_primitive(system:'$skip_list'(_,_,_)).
+safe_primitive(system:'$seek_list'(_, _, _, _)).
                                         % attributes
 safe_primitive(system:get_attr(_,_,_)).
 safe_primitive(system:get_attrs(_,_)).
@@ -1099,16 +1100,16 @@ safe_output(current_error).
 
 :- public format_calls/3.                       % used in pengines_io
 
-format_calls(Format, _Args, _Calls) :-
-    var(Format),
-    !,
-    instantiation_error(Format).
 format_calls(Format, Args, Calls) :-
+    is_list(Args),
+    !,
     format_types(Format, Types),
     (   format_callables(Types, Args, Calls)
     ->  true
     ;   throw(error(format_error(Format, Types, Args), _))
     ).
+format_calls(Format, Arg, Calls) :-
+    format_calls(Format, [Arg], Calls).
 
 format_callables([], [], []).
 format_callables([callable|TT], [G|TA], [G|TG]) :-
