@@ -880,16 +880,6 @@ void PlStream::release()
   s_ = nullptr;
 }
 
-_SWI_CPP2_CPP_inline
-int
-PlStream::check_rc(int rc)
-{ if ( rc < 0 )
-  { release(); // throws an exception if stream has an error
-    throw PlUnknownError("Stream error");
-  }
-  return rc;
-}
-
 _SWI_CPP2_CPP_inline
 void
 PlStream::check_stream() const
@@ -897,27 +887,50 @@ PlStream::check_stream() const
     throw PlUnknownError("Stream not set");
 }
 
+/* JW: was using check_rc(), but somehown Apple Clang thinks ssize_t
+   maps both to int32_t and int64_t.
+ */
 #define _SWI_CPP2_CPP_check_rc(rc_t, defn, call) \
 _SWI_CPP2_CPP_inline \
 rc_t \
 PlStream::defn \
 { check_stream(); \
-  return check_rc(call); \
+  rc_t rc = call; \
+  if ( rc < 0 ) { release(); throw PlUnknownError("Stream error"); } \
+  return rc; \
+}
+
+#define _SWI_CPP2_CPP_check_brc(rc_t, defn, call) \
+_SWI_CPP2_CPP_inline \
+rc_t \
+PlStream::defn \
+{ check_stream(); \
+  rc_t rc = call; \
+  if ( !rc ) { release(); throw PlUnknownError("Stream error"); } \
+  return rc; \
+}
+
+#define _SWI_CPP2_CPP_nocheck(rc_t, defn, call) \
+_SWI_CPP2_CPP_inline \
+rc_t \
+PlStream::defn \
+{ check_stream(); \
+  return call; \
 }
 
 _SWI_CPP2_CPP_check_rc(int, set_timeout(int tmo), Sset_timeout(s_, tmo))
 _SWI_CPP2_CPP_check_rc(int, unit_size(), Sunit_size(s_));
-_SWI_CPP2_CPP_check_rc(bool, canrepresent(int c), Scanrepresent(c, s_))
+_SWI_CPP2_CPP_nocheck(bool, canrepresent(int c), Scanrepresent(c, s_))
 _SWI_CPP2_CPP_check_rc(int, putcode(int c), Sputcode(c, s_))
 _SWI_CPP2_CPP_check_rc(int, getcode(), Sgetcode(s_))
 _SWI_CPP2_CPP_check_rc(int, peekcode(), Speekcode(s_))
 _SWI_CPP2_CPP_check_rc(int, putw(int w), Sputw(w, s_))
 _SWI_CPP2_CPP_check_rc(int, getw(), Sgetw(s_))
-_SWI_CPP2_CPP_check_rc(int, read(void *data, size_t size, size_t elems), Sfread(data, size, elems, s_))
-_SWI_CPP2_CPP_check_rc(int, write(const void *data, size_t size, size_t elems), Sfwrite(data, size, elems, s_))
-_SWI_CPP2_CPP_check_rc(int, eof(), Sfeof(s_))
-_SWI_CPP2_CPP_check_rc(int, pasteof(), Sfpasteof(s_))
-_SWI_CPP2_CPP_check_rc(int, error(), Sferror(s_))
+_SWI_CPP2_CPP_nocheck(size_t, fread(void *data, size_t size, size_t elems), Sfread(data, size, elems, s_))
+_SWI_CPP2_CPP_nocheck(size_t, fwrite(const void *data, size_t size, size_t elems), Sfwrite(data, size, elems, s_))
+_SWI_CPP2_CPP_check_rc(int, feof(), Sfeof(s_))
+_SWI_CPP2_CPP_check_rc(int, fpasteof(), Sfpasteof(s_))
+_SWI_CPP2_CPP_check_rc(int, ferror(), Sferror(s_))
 _SWI_CPP2_CPP_check_rc(int, seterr(int which, const char *message), Sseterr(s_, which, message))
 _SWI_CPP2_CPP_check_rc(int, set_exception(term_t ex), Sset_exception(s_, ex))
 _SWI_CPP2_CPP_check_rc(int, setenc(IOENC new_enc, IOENC *old_enc), Ssetenc(s_, new_enc, old_enc))
@@ -929,8 +942,8 @@ _SWI_CPP2_CPP_check_rc(int, tell(), Stell(s_))
 _SWI_CPP2_CPP_check_rc(int, close(), Sclose(s_))
 _SWI_CPP2_CPP_check_rc(int, gcclose(int flags), Sgcclose(s_, flags))
 _SWI_CPP2_CPP_check_rc(ssize_t, read_pending(char *buf, size_t limit, int flags), Sread_pending(s_, buf, limit, flags))
-_SWI_CPP2_CPP_check_rc(size_t, pending(), Spending(s_))
-_SWI_CPP2_CPP_check_rc(int, puts(const char *q), Sfputs(q, s_))
+_SWI_CPP2_CPP_nocheck(size_t, pending(), Spending(s_))
+_SWI_CPP2_CPP_check_rc(int, fputs(const char *q), Sfputs(q, s_))
 _SWI_CPP2_CPP_check_rc(int, vprintf(const char *fm, va_list args), Svfprintf(s_, fm, args))
 _SWI_CPP2_CPP_check_rc(int, lock(), Slock(s_))
 _SWI_CPP2_CPP_check_rc(int, tryLock(), StryLock(s_))
@@ -940,16 +953,16 @@ _SWI_CPP2_CPP_check_rc(int64_t, tell64(), Stell64(s_))
 _SWI_CPP2_CPP_check_rc(int, seek64(int64_t pos, int whence), Sseek64(s_, pos, whence))
 _SWI_CPP2_CPP_check_rc(int, checkBOM(), ScheckBOM(s_))
 _SWI_CPP2_CPP_check_rc(int, writeBOM(), SwriteBOM(s_))
-_SWI_CPP2_CPP_check_rc(int, qlf_get_int64(int64_t *ip), PL_qlf_get_int64(s_, ip))
-_SWI_CPP2_CPP_check_rc(int, qlf_get_int32(int32_t *ip), PL_qlf_get_int32(s_, ip))
-_SWI_CPP2_CPP_check_rc(int, qlf_get_uint32(uint32_t *ip), PL_qlf_get_uint32(s_, ip))
-_SWI_CPP2_CPP_check_rc(int, qlf_get_double(double *fp), PL_qlf_get_double(s_, fp))
-_SWI_CPP2_CPP_check_rc(int, qlf_get_atom(atom_t *a), PL_qlf_get_atom(s_, a))
-_SWI_CPP2_CPP_check_rc(int, qlf_put_int64(int64_t i), PL_qlf_put_int64(i, s_))
-_SWI_CPP2_CPP_check_rc(int, qlf_put_int32(int32_t i), PL_qlf_put_int32(i, s_))
-_SWI_CPP2_CPP_check_rc(int, qlf_put_uint32(uint32_t i), PL_qlf_put_uint32(i, s_))
-_SWI_CPP2_CPP_check_rc(int, qlf_put_double(double f), PL_qlf_put_double(f, s_))
-_SWI_CPP2_CPP_check_rc(int, qlf_put_atom(atom_t a), PL_qlf_put_atom(a, s_))
+_SWI_CPP2_CPP_check_brc(bool, qlf_get_int64(int64_t *ip), PL_qlf_get_int64(s_, ip))
+_SWI_CPP2_CPP_check_brc(bool, qlf_get_int32(int32_t *ip), PL_qlf_get_int32(s_, ip))
+_SWI_CPP2_CPP_check_brc(bool, qlf_get_uint32(uint32_t *ip), PL_qlf_get_uint32(s_, ip))
+_SWI_CPP2_CPP_check_brc(bool, qlf_get_double(double *fp), PL_qlf_get_double(s_, fp))
+_SWI_CPP2_CPP_check_brc(bool, qlf_get_atom(atom_t *a), PL_qlf_get_atom(s_, a))
+_SWI_CPP2_CPP_check_brc(bool, qlf_put_int64(int64_t i), PL_qlf_put_int64(i, s_))
+_SWI_CPP2_CPP_check_brc(bool, qlf_put_int32(int32_t i), PL_qlf_put_int32(i, s_))
+_SWI_CPP2_CPP_check_brc(bool, qlf_put_uint32(uint32_t i), PL_qlf_put_uint32(i, s_))
+_SWI_CPP2_CPP_check_brc(bool, qlf_put_double(double f), PL_qlf_put_double(f, s_))
+_SWI_CPP2_CPP_check_brc(bool, qlf_put_atom(atom_t a), PL_qlf_put_atom(a, s_))
 
 _SWI_CPP2_CPP_inline
 void
@@ -995,7 +1008,7 @@ PlStream::printfX(const char *fm, ...)
 }
 
 
-
 #undef _SWI_CPP2_CPP_check_rc
+#undef _SWI_CPP2_CPP_nocheck
 
 #endif /*_SWI_CPP2_CPP*/
