@@ -1437,8 +1437,6 @@ public:
   // Implicit throw of PlExceptionFail if release fails:
   void release();
 
-  int check_rc(int rc); // check rc >= 0 else release() + throw exception
-
   void check_stream() const; // verify that stream is set
 
   // Following are simple wrappers around the S*() functions, which
@@ -1457,11 +1455,11 @@ public:
   int peekcode();
   int putw(int w);
   int getw();
-  int read(void *data, size_t size, size_t elems);
-  int write(const void *data, size_t size, size_t elems);
-  int eof();
-  int pasteof();
-  int error();
+  size_t fread(void *data, size_t size, size_t elems); // TODO: does not check for error
+  size_t fwrite(const void *data, size_t size, size_t elems); // TODO: does not check for error
+  int feof();
+  int fpasteof();
+  int ferror();
   void clearerr();
   int seterr(int which, const char *message);
   int set_exception(term_t ex);
@@ -1475,8 +1473,8 @@ public:
   int gcclose(int flags);
   char *gets(char *buf, int n);
   ssize_t read_pending(char *buf, size_t limit, int flags);
-  size_t pending();
-  int puts(const char *q);
+  size_t pending(); // TODO: does not check for error
+  int fputs(const char *q);
   int printf(const char *fm, ...) WPRINTF23;
   int printfX(const char *fm, ...);
   int vprintf(const char *fm, va_list args);
@@ -1493,16 +1491,16 @@ public:
   int	checkBOM();
   int	writeBOM();
 
-  int qlf_get_int64(int64_t *ip);
-  int qlf_get_int32(int32_t *ip);
-  int qlf_get_uint32(uint32_t *ip);
-  int qlf_get_double(double *fp);
-  int qlf_get_atom(atom_t *a);
-  int qlf_put_int64(int64_t i);
-  int qlf_put_int32(int32_t i);
-  int qlf_put_uint32(uint32_t i);
-  int qlf_put_double(double f);
-  int qlf_put_atom(atom_t a);
+  bool qlf_get_int64(int64_t *ip);
+  bool qlf_get_int32(int32_t *ip);
+  bool qlf_get_uint32(uint32_t *ip);
+  bool qlf_get_double(double *fp);
+  bool qlf_get_atom(atom_t *a);
+  bool qlf_put_int64(int64_t i);
+  bool qlf_put_int32(int32_t i);
+  bool qlf_put_uint32(uint32_t i);
+  bool qlf_put_double(double f);
+  bool qlf_put_atom(atom_t a);
 
 private:
   IOSTREAM* s_ = nullptr;
