@@ -64,7 +64,7 @@
           ]).
 :- autoload(library(apply),[foldl/4,maplist/3,maplist/4]).
 :- autoload(library(backcomp),[thread_at_exit/1]).
-:- autoload(library(debug),[assertion/1]).
+:- use_module(library(debug),[assertion/1]).
 :- autoload(library(error),[must_be/2]).
 :- autoload(library(listing),[listing/1,portray_clause/1]).
 :- autoload(library(lists),[append/2,append/3,subtract/3]).
@@ -165,9 +165,10 @@ pengine_nl :-
 %
 %   Emit N spaces
 
-pengine_tab(N) :-
+pengine_tab(Expr) :-
     pengine_output,
     !,
+    N is Expr,
     length(List, N),
     maplist(=(&(nbsp)), List),
     send_html(List).
@@ -326,10 +327,27 @@ message_lines([ansi(Attributes, Fmt, Args)|T]) -->
     },
     html(HTML),
     message_lines(T).
+message_lines([url(Pos)|T]) -->
+    !,
+    location(Pos),
+    message_lines(T).
+message_lines([url(HREF, Label)|T]) -->
+    !,
+    html(a(href(HREF),Label)),
+    message_lines(T).
 message_lines([H|T]) -->
     html(H),
     message_lines(T).
 
+location(File:Line:Column) -->
+    !,
+    html([File, :, Line, :, Column]).
+location(File:Line) -->
+    !,
+    html([File, :, Line]).
+location(File) -->
+    html([File]).
+
 style(bold, Content, b(Content)) :- !.
 style(fg(default), Content, span(style('color: black'), Content)) :- !.
 style(fg(Color), Content, span(style('color:'+Color), Content)) :- !.
