@@ -237,8 +237,8 @@ summary(String, MaxLen, Summary) :-
     ).
 
 
-%!  clause_source(+Clause, -File, -Line) is det.
-%!  clause_source(-Clause, +File, -Line) is det.
+%!  clause_source(+Clause, -File, -Line) is semidet.
+%!  clause_source(-Clause, +File, -Line) is semidet.
 
 clause_source(Clause, File, Line) :-
     nonvar(Clause),
@@ -266,9 +266,11 @@ list_details(File, Options) :-
     memberchk(M, Modules),
     !.
 list_details(File, Options) :-
-    (   source_file_property(File, module(M))
-    ->  module_property(M, class(user))
-    ;   true     % non-module file must be user file.
+    (   source_file_property(File, module(M)),
+        module_property(M, class(user))
+    ->  true
+    ;   forall(source_file_property(File, module(M)),
+               module_property(M, class(test)))
     ),
     annotate_file(Options).
 
