@@ -166,12 +166,12 @@ DllMain(HINSTANCE hinstDll, DWORD fdwReason, LPVOID lpvReserved)
 #ifdef _MSC_VER
 /* Only for our purpose */
 static inline int
-__sync_bool_compare_and_swap(volatile void **addr, void *new, void *old)
+__sync_bool_compare_and_swap(volatile void **addr, void *old, void *new)
 {
 #if _WIN64
-    return (int)_InterlockedCompareExchange64((volatile __int64*)addr, (__int64)new, (__int64)old);
+  return _InterlockedCompareExchange64((volatile __int64*)addr, (__int64)new, (__int64)old) == (__int64)old;
 #else
-    return (int)_InterlockedCompareExchange64((volatile long*)addr, (long)new, (long)old);
+  return _InterlockedCompareExchange64((volatile long*)addr, (long)new, (long)old) == (long)old;
 #endif
 }
 
@@ -371,10 +371,10 @@ get_py_obj(term_t t, PyObject **objp, int error)
 }
 
 static int
-get_py_name(term_t name, PyObject **obj)
+get_py_name(term_t name, PyObject **obj, int flags)
 { char *idname;
 
-  if ( PL_get_chars(name, &idname, CVT_ATOM) )
+  if ( PL_get_chars(name, &idname, CVT_ATOM|flags) )
   { PyObject *id = check_error(PyUnicode_FromString(idname));
 
     if ( id )
@@ -388,6 +388,34 @@ get_py_name(term_t name, PyObject **obj)
 
 static htable *py_module_table = NULL;
 
+static int
+py_register_module(term_t name, PyObject **mod, int flags)
+{ PyObject *idobj;
+  atom_t id;
+
+  if ( !py_module_table )
+    py_module_table = py_new_hashmap();
+
+  if ( get_py_name(name, &idobj, flags) &&
+       PL_get_atom(name, &id) )
+  { PyObject *m = check_error(PyImport_Import(idobj));
+
+    Py_DECREF(idobj);
+    if ( m )
+    { PyObject *old = NULL;
+      if ( mod )
+	*mod = m;
+      int rc = py_add_hashmap(py_module_table, id, m, &old);
+      if ( old )
+	Py_DECREF(old);
+      return rc;
+    }
+  }
+
+  return FALSE;
+}
+
+
 static int
 get_py_module(term_t name, PyObject **mod)
 { atom_t id;
@@ -410,17 +438,9 @@ get_py_module(term_t name, PyObject **mod)
       *mod = obj;
       return TRUE;
     } else
-    { PyObject *idobj;
-
-      if ( get_py_name(name, &idobj) )
-      { PyObject *m = check_error(PyImport_Import(idobj));
-
-	Py_DECREF(idobj);
-	if ( m )
-	{ *mod = m;
-	  py_add_hashmap(py_module_table, id, m);
-	  return TRUE;
-	}
+    { if ( py_register_module(name, mod, 0) )
+      { Py_INCREF(*mod);
+	return TRUE;
       }
     }
   }
@@ -429,6 +449,20 @@ get_py_module(term_t name, PyObject **mod)
 }
 
 
+static foreign_t
+py_update_module_cache(term_t name)
+{ int rc;
+  py_gil_state state;
+
+  if ( !py_gil_ensure(&state) )
+    return FALSE;
+  rc = py_register_module(name, NULL, CVT_EXCEPTION);
+  py_gil_release(state);
+
+  return rc;
+}
+
+
 static PyObject *
 check_error(PyObject *obj)
 { PyObject *ex = PyErr_Occurred();
@@ -1817,6 +1851,8 @@ get_conversion_options(term_t options, int *flags)
   return TRUE;
 }
 
+static int said_deprecated_setattr = FALSE;
+
 static foreign_t
 py_call3(term_t Call, term_t result, term_t options)
 { PyObject *py_target = NULL;
@@ -1833,6 +1869,11 @@ py_call3(term_t Call, term_t result, term_t options)
   { val = PL_new_term_ref();
     _PL_get_arg(2, call, val);
     _PL_get_arg(1, call, call);
+    if ( !said_deprecated_setattr )
+    { said_deprecated_setattr = TRUE;
+      Sdprintf("Deprecated: instead of py_call(Obj:Attr = Value), "
+	       "use py_setattr(Obj,Attr,Value)");
+    }
   }
 
   if ( !py_gil_ensure(&state) )
@@ -1884,6 +1925,43 @@ py_call1(term_t Call)
 { return py_call3(Call, 0, 0);
 }
 
+static foreign_t
+py_setattr(term_t On, term_t name, term_t value)
+{ term_t on = PL_copy_term_ref(On);
+  PyObject *py_target = NULL;
+  int rc;
+  py_gil_state state;
+
+  if ( !py_gil_ensure(&state) )
+    return FALSE;
+
+  rc = unchain(on, &py_target);
+  PyObject *next = py_eval(py_target, on);
+  Py_XDECREF(py_target);
+  py_target = next;
+  rc = !!py_target;
+
+  if ( rc )
+  { char *attr;
+
+    PL_STRINGS_MARK();
+    if ( (rc=PL_get_chars(name, &attr, CVT_ATOM|REP_UTF8|CVT_EXCEPTION)) )
+    { PyObject *py_val = NULL;
+
+      if ( (rc=py_from_prolog(value, &py_val)) )
+      { if ( PyObject_SetAttrString(py_target, attr, py_val) == -1 )
+	  rc = !!check_error(NULL);
+      }
+      Py_CLEAR(py_val);
+    }
+    PL_STRINGS_RELEASE();
+  }
+
+  Py_CLEAR(py_target);
+  py_gil_release(state);
+
+  return rc;
+}
 
 typedef struct
 { PyObject *iterator;
@@ -2261,21 +2339,24 @@ install_janus(void)
 
 #define REGISTER(name, arity, func, flags) \
         PL_register_foreign_in_module("janus", name, arity, func, flags)
+#define NDET PL_FA_NONDETERMINISTIC
 
-  REGISTER("py_initialize_",  3, py_initialize_, 0);
-  REGISTER("py_finalize",     0, py_finalize,    0);
-  REGISTER("py_call",         1, py_call1,       0);
-  REGISTER("py_call",         2, py_call2,       0);
-  REGISTER("py_call",         3, py_call3,       0);
-  REGISTER("py_iter",         2, py_iter2,       PL_FA_NONDETERMINISTIC);
-  REGISTER("py_iter",         3, py_iter3,       PL_FA_NONDETERMINISTIC);
-  REGISTER("py_run",          5, py_run,         0);
-  REGISTER("py_free",         1, py_free,        0);
-  REGISTER("py_is_object",    1, py_is_object,   0);
-  REGISTER("py_with_gil",     1, py_with_gil,    PL_FA_TRANSPARENT);
-  REGISTER("py_gil_owner",    1, py_gil_owner,   0);
-  REGISTER("py_str",          2, py_str,         0);
-  REGISTER("py_debug",        1, py_debug,       0);
+  REGISTER("py_initialize_",	     3,	py_initialize_,		0);
+  REGISTER("py_finalize",	     0,	py_finalize,		0);
+  REGISTER("py_call",		     1,	py_call1,		0);
+  REGISTER("py_call",		     2,	py_call2,		0);
+  REGISTER("py_call",		     3,	py_call3,		0);
+  REGISTER("py_iter",		     2,	py_iter2,		NDET);
+  REGISTER("py_iter",		     3,	py_iter3,		NDET);
+  REGISTER("py_setattr",	     3, py_setattr,		0);
+  REGISTER("py_run",		     5,	py_run,			0);
+  REGISTER("py_free",		     1,	py_free,		0);
+  REGISTER("py_is_object",	     1,	py_is_object,		0);
+  REGISTER("py_with_gil",	     1,	py_with_gil,		NDET);
+  REGISTER("py_gil_owner",	     1,	py_gil_owner,		0);
+  REGISTER("py_str",		     2,	py_str,			0);
+  REGISTER("py_debug",		     1,	py_debug,		0);
+  REGISTER("py_update_module_cache", 1,	py_update_module_cache,	0);
 
   if ( PyImport_AppendInittab("_swipl", PyInit__swipl) == -1 )
     Sdprintf("Failed to add module swipl to Python");
