@@ -40,6 +40,7 @@
             py_call/3,                  % +Call, -Return, +Options
 	    py_iter/2,			% +Call, -Return
 	    py_iter/3,			% +Call, -Return, +Options
+            py_setattr/3,               % +On, +Name, +Value
             py_free/1,			% +Obj
 	    py_is_object/1,		% @Term
 	    py_with_gil/1,		% :Goal
@@ -64,6 +65,8 @@
             py_obj_dir/2,               % +ObjRef,-List
             py_obj_dict/2,              % obj_dict(+ObjRef, -Dict)
 
+            py_module/2,                % +Module:atom, +Source:string
+
             py_initialize/3,            % +Program, +Argv, +Options
             py_lib_dirs/1,              % -Dirs
             py_add_lib_dir/0,           % (directive)
@@ -217,15 +220,6 @@ py_version :-
 %       Dog = <py_Dog>(0x7f095c9d02e0),
 %       Tricks = ["roll_over"]
 %
-%   py_call/1 can also be used to set an attribute on a module or object
-%   using the syntax py_call(Obj:Attr = Value). For example:
-%
-%       ?- py_call(dog:'Dog'("Fido"), Dog),
-%          py_call(Dog:owner = "Bob"),
-%          py_call(Doc:owner, Owner).
-%       Dog = <py_Dog>(0x7ffff7112170),
-%       Owner = "Bob".
-%
 %   If the principal term of the   first  argument is not `Target:Func`,
 %   The argument is evaluated as the initial target, i.e., it must be an
 %   object reference or a module.   For example:
@@ -291,6 +285,17 @@ py_version :-
 %   @arg Options is processed as with py_call/3.
 %   @bug Iterator may not depend on janus.query()
 
+%!  py_setattr(+Target, +Name, +Value) is det.
+%
+%   Set a Python attribute on an object.  If   Target  is an atom, it is
+%   interpreted  as  a  module.  Otherwise  it  is  normally  an  object
+%   reference. py_setattr/3 allows for  _chaining_   and  behaves  as if
+%   defined as
+%
+%       py_setattr(Target, Name, Value) :-
+%           py_call(Target, Obj, [py_object(true)]),
+%           py_call(setattr(Obj, Name, Value)).
+
 %!  py_run(+String, +Globals, +Locals, -Result, +Options) is det.
 %
 %   Interface  to  Py_CompileString()  followed   by  PyEval_EvalCode().
@@ -306,7 +311,7 @@ py_version :-
 
 %!  py_is_object(@Term) is semidet.
 %
-%   True when Term is a Python object reference. Fails silently if @Term
+%   True when Term is a Python object reference. Fails silently if Term
 %   is any other Prolog term.
 %
 %   @error existence_error(py_object, Term) is raised of Term is a
@@ -574,6 +579,43 @@ py_obj_dict(ObjRef, Dict) :-
     py_call(ObjRef:'__dict__', Dict).
 
 
+%!  py_module(+Module:atom, +Source:string) is det.
+%
+%   Load Source into the Python module Module.   This  is intended to be
+%   used together with the `string` _quasi quotation_ that supports long
+%   strings in SWI-Prolog.   For example:
+%
+%   ```
+%   :- use_module(library(strings)).
+%   :- py_module(hello,
+%                {|string||
+%                 | def say_hello_to(s):
+%                 |     print(f"hello {s}")
+%                 |}).
+%   ```
+%
+%   Calling this predicate multiple  times  with   the  same  Module and
+%   Source is a no-op. Called with  a   different  source  creates a new
+%   Python module that replaces the old in the global namespace.
+%
+%   @error python_error(Type, Data, Stack) is raised if Python raises an
+%   error.
+
+:- dynamic py_dyn_module/2 as volatile.
+
+py_module(Module, Source) :-
+    variant_sha1(Source, Hash),
+    (   py_dyn_module(Module, Hash)
+    ->  true
+    ;   py_call(janus:import_module_from_string(Module, Source)),
+        (   retract(py_dyn_module(Module, _))
+        ->  py_update_module_cache(Module)
+        ;   true
+        ),
+        asserta(py_dyn_module(Module, Hash))
+    ).
+
+
 		 /*******************************
 		 *            INIT		*
 		 *******************************/
@@ -636,7 +678,7 @@ no_env_dir('dist-packages').
 %   list.
 %
 %   Calling this predicate while the  Python is already initialized is
-%   a no-op.   This predicate is  thread-safe, where the  first thread
+%   a  no-op.  This  predicate is  thread-safe, where  the first  call
 %   initializes Python.
 %
 %   In addition to initializing the Python system, it
