@@ -39,7 +39,7 @@ janus`.   The module provides three groups of support:
 
   - Class janus.query and function janus.query_once() that allow calling
     Prolog at a high level of abstraction.
-  - Functions janus.px_cmp(), janus.qdet() and janus.comp() that
+  - Functions janus.cmp(), janus.qdet() and janus.comp() that
     provide a more low-level interface for calling Prolog that
     is compatible with the original version of Janus for XSB.
   - The classes `Term` and `PrologError` to represent arbitrary
@@ -372,12 +372,26 @@ def prolog():
     query_once("'$toplevel':setup_interactive")
     query_once("prolog")
 
+import sys, importlib.util
+
+# from https://stackoverflow.com/a/53080237/717069
+def import_module_from_string(name: str, source: str):
+    """
+    Import module from source string.
+    Example use:
+    import_module_from_string("m", "f = lambda: print('hello')")
+    m.f()
+    """
+    spec = importlib.util.spec_from_loader(name, loader=None)
+    module = importlib.util.module_from_spec(spec)
+    exec(source, module.__dict__)
+    sys.modules[name] = module
+    globals()[name] = module
+
 ################################################################
 # Emulated XSB interface
 
-def px_cmd(module, pred, *args):
-    """Run module:pred(arg ...)"""
-    query_once("janus:px_cmd(M,P,Args)", {"M":module, "P":pred, "Args":args})
+cmd = _swipl.cmd
 
 def _xsb_tv(truth):
     if truth == True:
@@ -387,8 +401,8 @@ def _xsb_tv(truth):
     else:
         return 2
 
-def px_qdet(module, pred, *args):
-    """Run predicate as query_once/1
+def qdet(module, pred, *args):
+    """Run predicate as apply_once/1
 
     The predicate is called with one more argument than provided in `args`.
 
@@ -396,11 +410,11 @@ def px_qdet(module, pred, *args):
     --------
     **Example 1**
 
-    >>> px_qdet("user", "current_prolog_flag", "version")
+    >>> qdet("user", "current_prolog_flag", "version")
     (90113, 1)
 
     **Example 2**
-    >>> px_qdet("user", "current_prolog_flag", "no_such_flag")
+    >>> qdet("user", "current_prolog_flag", "no_such_flag")
     (None, 0)
 
     Returns
@@ -413,14 +427,14 @@ def px_qdet(module, pred, *args):
     d = query_once("janus:px_call(Args,M,P,Ret)", {"M":module, "P":pred, "Args":args})
     return (d["Ret"], _xsb_tv(d["truth"]))
 
-def px_comp(module, pred, *args, vars=1, set_collect=False, truth=TruthVal.PLAIN_TRUTHVALS):
+def comp(module, pred, *args, vars=1, set_collect=False, truth=TruthVal.PLAIN_TRUTHVALS):
     """Call non-deterministic predicate
 
 
     Examples
     --------
 
-    >>> px_comp("user", "between", 1, 6)
+    >>> comp("user", "between", 1, 6)
     [((1,), 1), ((2,), 1), ((3,), 1), ((4,), 1), ((5,), 1), ((6,), 1)]
 
     Parameters
