@@ -61,6 +61,7 @@ test_janus :-
 		janus_iter,
 		janus_errors,
 		janus_unicode,
+                janus_load,
 		xsb_call
               ]).
 
@@ -124,7 +125,7 @@ test(set, Set == [1, a, false]) :- % True canot be in a Python set??
     py_call(janus:echo(py_set([1,a,false])), py_set(List)),
     sort(List, Set).
 test(attr, Val = 42) :-
-    py_call(demo:test_attr = 42),
+    py_setattr(demo, test_attr, 42),
     py_call(demo:test_attr, Val).
 test(stringify, R == '6') :-
     py_call(janus:echo(#6), R).
@@ -255,7 +256,7 @@ test(arg, R == py{a:a,b:b,c:c}) :-
 test(gc1) :-
     current_prolog_flag(gc_thread, Old),
     set_prolog_gc_thread(false),
-    py_call(demo:gced = 0),
+    py_setattr(demo, gced, 0),
     forall(between(1, 10 000, _),
            py_call(demo:'GCAble'(), _)),
     garbage_collect_atoms,
@@ -266,7 +267,7 @@ test(gc2, [GCed0,GCed] == [10 000, 10 000]) :-
     current_prolog_flag(gc_thread, Old),
     set_prolog_gc_thread(false),
     garbage_collect_atoms,
-    py_call(demo:gced = 0),
+    py_setattr(demo, gced, 0),
     forall(between(1, 10 000, _),
            ( py_call(demo:'GCAble'(), Obj),
 	     py_free(Obj) )),
@@ -275,7 +276,7 @@ test(gc2, [GCed0,GCed] == [10 000, 10 000]) :-
     set_prolog_gc_thread(Old),
     py_call(demo:gced, GCed).
 test(free, GCed == 1) :-
-    py_call(demo:gced = 0),
+    py_setattr(demo, gced, 0),
     py_call(demo:'GCAble'(), Obj),
     py_free(Obj),
     py_call(demo:gced, GCed).
@@ -286,7 +287,7 @@ test(free, error(existence_error('PyObject', Obj))) :-
 test(free, error(existence_error('PyObject', Obj))) :-
     py_call(demo:'GCAble'(), Obj),
     py_free(Obj),
-    py_call(Obj:test = 1).
+    py_setattr(Obj, test, 1).
 
 :- end_tests(janus_gc).
 
@@ -366,21 +367,62 @@ test(iso_latin_1, R == schloß) :-
 
 :- end_tests(janus_unicode).
 
+:- begin_tests(janus_load).
+:- use_module(library(strings)).
+
+test(load, X == 4) :-
+    py_module(load_test,
+              {|string||
+               | def times_two(n):
+               |     return n*2
+               |}),
+    py_call(load_test:times_two(2), X).
+test(reload, [X1,X2] == [6,4]) :-
+    py_module(load_test2,
+              {|string||
+               | def times_two(n):
+               |     return n*3
+               |}),
+    py_call(load_test2:times_two(2), X1),
+    py_module(load_test2,
+              {|string||
+               | def times_two(n):
+               |     return n*2
+               |}),
+    py_call(load_test2:times_two(2), X2).
+
+:- end_tests(janus_load).
+
+
 :- begin_tests(xsb_call).
 
 test(reverse, X == -([py{a:py{b:c}}, -(mytuple), 3, 2, 1], 1)) :-
-    py_call(janus:px_qdet(lists, reverse,
-			  [1,2,3,-(mytuple),py{a:py{b:c}}]),
+    py_call(janus:qdet(lists, reverse,
+		       [1,2,3,-(mytuple),py{a:py{b:c}}]),
 	    X).
-test(px_comp1, X == [-(1)-1,-(2)-1]) :-
-    py_call(janus:px_comp(user, between, 1, 2), X).
-test(px_comp2, X == [-(1),-(2)]) :-
+test(comp1, X == [-(1)-1,-(2)-1]) :-
+    py_call(janus:comp(user, between, 1, 2), X).
+test(comp2, X == [-(1),-(2)]) :-
     py_call(janus:'NO_TRUTHVALS', NoThruthVals),
-    py_call(janus:px_comp(user, between, 1, 2, truth=NoThruthVals), X).
+    py_call(janus:comp(user, between, 1, 2, truth=NoThruthVals), X).
+test(cmd, X == @(true)) :-
+    py_call(janus:cmd(user, true), X).
+test(cmd, X == @(false)) :-
+    py_call(janus:cmd(user, fail), X).
+test(cmd, X == Undefined) :-
+    py_call(janus:undefined, Undefined),
+    py_call(janus:cmd(user, undefined), X).
+test(cmd, X == @true) :-
+    py_call(janus:cmd(test_janus, p, 42), X).
+test(cmd, X == @false) :-
+    py_call(janus:cmd(test_janus, p, 1), X).
+test(cmd, error(python_error('PrologError', _, _))) :-
+    py_call(janus:cmd(test_janus, no_such_pred), _).
+
+test_janus:p(42).
 
 :- end_tests(xsb_call).
 
-
 bench_janus :-
     bench_python(concat_list(100 000)).
 
