@@ -3,8 +3,9 @@
     Author:        Jan Wielemaker and Keri Harris
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2011-2016, University of Amsterdam
+    Copyright (c)  2011-2023, University of Amsterdam
                               VU University Amsterdam
+                              SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -69,23 +70,33 @@ struct table_enum
   int		idx;		/* Index of current symbol-chain */
 };
 
-COMMON(void)		initTables(void);
-COMMON(Table)		newHTable(int size);
-COMMON(void)		destroyHTable(Table ht);
-COMMON(void*)		lookupHTable__LD(Table ht, void *name ARG_LD);
-COMMON(void*)		addHTable(Table ht, void *name, void *value);
-COMMON(void*)		addNewHTable(Table ht, void *name, void *value);
-COMMON(void*)		updateHTable(Table ht, void *name, void *value);
-COMMON(void*)		deleteHTable(Table ht, void *name);
-COMMON(void)		clearHTable(Table ht);
-COMMON(Table)		copyHTable(Table org);
-COMMON(TableEnum)	newTableEnum(Table ht);
-COMMON(void)		freeTableEnum(TableEnum e);
-COMMON(int)		advanceTableEnum(TableEnum e, void **name, void **value);
+#if USE_LD_MACROS
+#define	lookupHTable(ht, name)		LDFUNC(lookupHTable, ht, name)
+#define	addHTable(ht, name, value)	LDFUNC(addHTable, ht, name, value)
+#define	addNewHTable(ht, name, value)	LDFUNC(addNewHTable, ht, name, value)
+#endif /*USE_LD_MACROS*/
+
+#define LDFUNC_DECLARATIONS
+
+void		initTables(void);
+Table		newHTable(int size);
+void		destroyHTable(Table ht);
+void*		lookupHTable(Table ht, void *name);
+void*		addHTable(Table ht, void *name, void *value);
+void*		addNewHTable(Table ht, void *name, void *value);
+void*		updateHTable(Table ht, void *name, void *value);
+void*		deleteHTable(Table ht, void *name);
+void		clearHTable(Table ht);
+Table		copyHTable(Table org);
+TableEnum	newTableEnum(Table ht);
+void		freeTableEnum(TableEnum e);
+int		advanceTableEnum(TableEnum e, void **name, void **value);
 					/* used by for_table() macro */
-COMMON(int)		htable_iter(Table ht, KVS kvs, int *idx,
-				    void **name, void **value);
-COMMON(size_t)		sizeofTable(Table ht);
+int		htable_iter(Table ht, KVS kvs, int *idx,
+			    void **name, void **value);
+size_t		sizeofTable(Table ht);
+
+#undef LDFUNC_DECLARATIONS
 
 static inline int
 htable_valid_kv(void *kv)
@@ -140,10 +151,4 @@ htable_valid_kv(void *kv)
           ATOMIC_DEC(&kvs->accesses); \
 	}
 
-		 /*******************************
-		 *	LD-USING FUNCTIONS	*
-		 *******************************/
-
-#define lookupHTable(ht, n)	lookupHTable__LD(ht, n PASS_LD)
-
 #endif /*TABLE_H_INCLUDED*/
