@@ -390,6 +390,14 @@ struct PL_global_data
     double	cgc_clause_factor;	/* Pce to scan clauses */
   } clauses;
 
+  struct
+  { float	min_speedup;
+    float	max_var_fraction;
+    float	min_speedup_ratio;
+    int		max_lookahead;
+    int		min_clauses;
+  } clause_index;
+
   struct
   { size_t	highest;		/* highest source file index */
     size_t	no_hole_before;		/* All filled before here */
@@ -482,7 +490,6 @@ struct PL_local_data
   int		break_level;		/* current break level */
   Stack		outofstack;		/* thread is out of stack */
   int		trim_stack_requested;	/* perform a trim-stack */
-  int		in_arithmetic;		/* doing arithmetic */
   int		in_print_message;	/* Inside printMessage() */
   void *	glob_info;		/* pl-glob.c */
   IOENC		encoding;		/* default I/O encoding */
@@ -675,7 +682,7 @@ struct PL_local_data
     int	has_scheduling_component;	/* A leader was created */
     int in_answer_completion;		/* Running answer completion */
     int in_assert_propagation;		/* Running propagate_assert/1 */
-    int flags;				/* Global flags (TF_*) */
+    unsigned int flags;			/* Global flags (TF_*) */
     term_t delay_list;			/* Global delay list */
     term_t idg_current;			/* Current node in IDG (trie symbol) */
     struct
