@@ -4,7 +4,7 @@
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
     Copyright (c)  2012-2022, University of Amsterdam
-                              VU University Amsterdam
+			      VU University Amsterdam
 			      CWI, Amsterdam
 			      SWI-Prolog Solutions b.v.
     All rights reserved.
@@ -277,7 +277,9 @@ findHome(const char *symbols, int argc, const char **argv)
     IOSTREAM *fd;
     char *pparent;
 
-    if ( !(pparent=DirName(DirName(AbsoluteFile(maybe_home,parent),parent),parent)) ||
+    if ( !(pparent=DirName(DirName(AbsoluteFile(maybe_home,parent,sizeof(parent)),
+				   parent),
+			   parent)) ||
 	 strlen(PLHOMEFILE) + 1 + strlen(pparent) + 1 > sizeof(parent) )
       fatalError("File name too long: %s", home);
 
@@ -302,10 +304,10 @@ findHome(const char *symbols, int argc, const char **argv)
 	{ char buf2[PATH_MAX];
 
 	  if ( Ssnprintf(buf2, sizeof(buf2), "%s/%s", parent, buf) < 0 ||
-	       !(maybe_home = AbsoluteFile(buf2, plp)) )
+	       !(maybe_home = AbsoluteFile(buf2, plp, sizeof(plp))) )
 	    fatalError("Path name too long: %s/%s", parent, buf);
 	} else
-	{ if ( !(maybe_home = AbsoluteFile(buf, plp)) )
+	{ if ( !(maybe_home = AbsoluteFile(buf, plp, sizeof(plp))) )
 	    fatalError("Path name too long: %s/%s", buf);
 	}
 
@@ -437,11 +439,15 @@ on_error_style(const char *s)
 }
 
 
+PL_EXPORT_DATA(int) plugin_is_GPL_compatible;
 int plugin_is_GPL_compatible;
 
+PL_EXPORT(int) emacs_module_init(void*a);
+
 int
-emacs_module_init(void*a) {
-  (void)a;
+emacs_module_init(void*a)
+{ (void)a;
+
   return 0;
 }
 
@@ -637,7 +643,7 @@ again:
 	return FALSE;
       case '=':
 	addBuffer(name, EOS, char);
-        goto do_value;
+	goto do_value;
       case '\n':
 	discardBuffer(name);
 	goto again;
@@ -652,7 +658,7 @@ do_value:
     { case EOF:
       case '\n':
 	addBuffer(value, EOS, char);
-        return TRUE;
+	return TRUE;
       default:
 	addBuffer(value, c, char);
     }
@@ -763,7 +769,6 @@ parseCommandLineOptions(int argc0, char **argv0, char **argvleft, int compile)
       { if ( rc == TRUE )
 	{ if ( !b )
 	  { clearPrologFlagMask(PLFLAG_SIGNALS);
-	    clearPrologFlagMask(PLFLAG_GCTHREAD);
 	    GD->options.nosignals = TRUE;
 	  }
 	} else
@@ -860,6 +865,17 @@ parseCommandLineOptions(int argc0, char **argv0, char **argvleft, int compile)
       continue;
     }
 
+    if ( *s == 'D' )
+    { const char *def = s+1;
+
+      if ( *def )
+	opt_append(&GD->options.defines, def);
+      else
+	optionList(&GD->options.defines);
+
+      continue;
+    }
+
     while(*s)
     { switch(*s)
       { case 'd':	if ( argc > 1 )
@@ -890,7 +906,7 @@ parseCommandLineOptions(int argc0, char **argv0, char **argvleft, int compile)
 	case 'b':	break;			/* already processed */
 	case 'q':	GD->options.silent = TRUE;
 			break;
-        default:
+	default:
 	{ if ( s == &argv[0][1] )
 	  { argvleft[argcleft++] = argv[0];
 	    goto next;
@@ -1178,7 +1194,6 @@ PL_initialise(int argc, char **argv)
   if ( GD->options.nosignals )
   { GET_LD
     clearPrologFlagMask(PLFLAG_SIGNALS);
-    clearPrologFlagMask(PLFLAG_GCTHREAD);
   }
 
   if ( GD->bootsession )
@@ -1284,6 +1299,7 @@ usage(void)
     "    -l file                  Script source file\n",
     "    -s file                  Script source file\n",
     "    -p alias=path            Define file search path 'alias'\n",
+    "    -D name=value		  Set a Prolog flag\n",
     "    -O                       Optimised compilation\n",
     "    --on-error=style         One of print, halt or status\n",
     "    --on-warning=style       One of print, halt or status\n",
@@ -1304,7 +1320,7 @@ usage(void)
     "    --pce[=bool]             Make the xpce gui available\n",
     "    --pldoc[=port]           Start PlDoc server [at port]\n",
 #ifdef __WINDOWS__
-    "    --win-app	          Behave as Windows application\n",
+    "    --win-app		  Behave as Windows application\n",
 #endif
 #ifdef O_DEBUG
     "    -d topic,topic,...       Enable C-source DEBUG channels\n",
@@ -1477,6 +1493,8 @@ PL_cleanup(int status)
   if ( GD->cleaning != CLN_NORMAL )
     return PL_CLEANUP_RECURSIVE;
 
+  checkPrologFlagsAccess();
+
 #ifdef __WINDOWS__
   if ( rval != 0 && !hasConsole() )
     PlMessage("Exit status is %d", rval);
@@ -1508,7 +1526,7 @@ PL_cleanup(int status)
     if ( LD->outofstack )
       emptyStacks();
 
-    PL_set_prolog_flag("exit_status", PL_INTEGER, rval);
+    PL_set_prolog_flag("exit_status", PL_INTEGER, (intptr_t)rval);
     if ( query_loop(PL_new_atom("$run_at_halt"), FALSE) == FALSE &&
 	 !(status&PL_CLEANUP_NO_CANCEL) )
     { if ( ++GD->halt_cancelled	< MAX_HALT_CANCELLED )
@@ -1560,7 +1578,7 @@ emergency:
     predicate_t proc = PL_predicate("unload_all_foreign_libraries", 0,
 				    "shlib");
     if ( isDefinedProcedure(proc) )
-      PL_call_predicate(MODULE_system, FALSE, proc, 0);
+      PL_call_predicate(MODULE_system, PL_Q_NODEBUG, proc, 0);
     PL_discard_foreign_frame(cid);
   }
 
@@ -1714,8 +1732,8 @@ vsysError(const char *fm, va_list args)
   if ( PL_get_thread_alias(tid, &alias) )
     name = PL_atom_wchars(alias, NULL);
 
-  Sfprintf(Serror, "[PROLOG SYSTEM ERROR:  Thread %d (%Ws) at %s\n\t",
-	   tid, name, tbuf);
+  SfprintfX(Serror, "[PROLOG SYSTEM ERROR:  Thread %d (%Ws) at %s\n\t",
+	    tid, name, tbuf);
 }
 #else
   Sfprintf(Serror, "[PROLOG SYSTEM ERROR: at %s\n\t", tbuf);
@@ -1865,11 +1883,11 @@ vwarning(const char *fm, va_list args)
     { nospace:
       if ( cid )
 	PL_discard_foreign_frame(cid);
-      Sfprintf(Suser_error, "ERROR: ");
+      Sfprintf(Suser_error, "%s", "ERROR: ");
       if ( s )
-        Sfprintf(Suser_error, s);
+	Sfprintf(Suser_error, "%s", s);
       else
-        Svfprintf(Suser_error, fm, args);
+	Svfprintf(Suser_error, fm, args);
       Sfprintf(Suser_error, "\n");
       Pause(0.2);
     }
