@@ -234,7 +234,7 @@ updateAlerted(PL_local_data_t *ld)
 #endif
   if ( ld->fli.string_buffers.top )		mask |= ALERT_BUFFER;
   if ( UNDO_SCHEDULED(ld) )			mask |= ALERT_UNDO;
-  if ( LD->coverage.active )			mask |= ALERT_COVERAGE;
+  if ( ld->coverage.active )			mask |= ALERT_COVERAGE;
 
   ld->alerted = mask;
 }
@@ -320,7 +320,7 @@ open_foreign_frame(DECL_LD)
   Mark(fr->mark);
   DEBUG(CHK_SECURE, assert(fr>fli_context));
   fr->parent = fli_context;
-  fr->magic = FLI_MAGIC;
+  FLI_SET_VALID(fr);
   fli_context = fr;
 
   return consTermRef(fr);
@@ -331,10 +331,10 @@ void
 PL_close_foreign_frame(DECL_LD fid_t id)
 { FliFrame fr = (FliFrame) valTermRef(id);
 
-  if ( !id || fr->magic != FLI_MAGIC )
+  if ( !id || !FLI_VALID(fr) )
     sysError("PL_close_foreign_frame(): illegal frame: %d", id);
   DiscardMark(fr->mark);
-  fr->magic = FLI_MAGIC_CLOSED;
+  FLI_SET_CLOSED(fr);
   fli_context = fr->parent;
   lTop = (LocalFrame) fr;
 }
@@ -383,7 +383,7 @@ PL_open_signal_foreign_frame(int sync)
   }
 
   fr = addPointer(lTop, margin);
-  fr->magic = FLI_MAGIC;
+  FLI_SET_VALID(fr);
   fr->size = 0;
   Mark(fr->mark);
   fr->parent = fli_context;
@@ -1431,7 +1431,9 @@ __do_undo(DECL_LD mark *m)
   while(--tt >= mt)
   { Word p = tt->address;
 
-    if ( isTrailVal(p) )
+    if ( likely(!isTrailVal(p)) )
+    { setVar(*p);
+    } else
     { DEBUG(2, Sdprintf("Undoing a trailed assignment\n"));
       tt--;
       if ( tt->address == gBase )
@@ -1440,9 +1442,8 @@ __do_undo(DECL_LD mark *m)
       DEBUG(CHK_SECURE,
 	    if ( isAttVar(*tt->address) )
 	      assert(on_attvar_chain(tt->address)));
-      assert(!(*tt->address & (MARK_MASK|FIRST_MASK)));
-    } else
-      setVar(*p);
+      DEBUG(0, assert(!(*tt->address & (MARK_MASK|FIRST_MASK))));
+    }
   }
 
   DEBUG(CHK_SECURE,
@@ -1452,15 +1453,8 @@ __do_undo(DECL_LD mark *m)
 
   tTop = mt;
 
-  Word ngtop;
-  if ( LD->frozen_bar > m->globaltop )
-  { DEBUG(CHK_SECURE, assert(gTop >= LD->frozen_bar));
-    reclaim_attvars(LD->frozen_bar);
-    ngtop = LD->frozen_bar;
-  } else
-  { reclaim_attvars(m->globaltop);
-    ngtop = m->globaltop;
-  }
+  Word ngtop = max(LD->frozen_bar, m->globaltop);
+  reclaim_attvars(ngtop);
 
   DEBUG(CHK_SECURE,
 	{ for(Word p = gTop; --p > ngtop;)
@@ -3183,7 +3177,8 @@ API_STUB(int)
 int
 PL_next_solution(DECL_LD qid_t qid)
 { register_file REGISTERS =
-  { .qid = qid
+  { .qid = qid,
+    .fndet_context.engine = LD
 #if VMCODE_IS_ADDRESS
     , .nop1 = 0, .nop2 = 0
 #endif
